@testset "gain" begin

# Create a system.
Ngrid = [3,3,3]
N = 3prod(Ngrid)
ind_in = 5:7
εc = ones(ComplexF64, N)
εc[ind_in] .= 12+0.1im
m = 1
ωₐ = 1.0
γ˔ = 1.0
D₀ = fill(0.01, N)
D₀[ind_in] .= 1.0

M = 4
ω = rand(M)
a² = rand(M)
Ψ = randn(ComplexF64,N,M)
ψ = [Ψ[:,j] for j = 1:M]

# Check automatic differentiation performed in the GainProfile constructor produces correct
# results.
gain = gen_gain_2lv(ωₐ, γ˔)
gp = GainProfile(gain, N)

gain′ = SALTBase.gen_gain′_2lv(ωₐ, γ˔)
abs2gain = SALTBase.gen_abs2gain_2lv(ωₐ, γ˔)
abs2gain′ = SALTBase.gen_abs2gain′_2lv(ωₐ, γ˔)
@test gp.gain′[1].(ω) ≈ gain′.(ω)
@test gp.abs2gain[1].(ω) ≈ abs2gain.(ω)
@test gp.abs2gain′[1].(ω) ≈ abs2gain′.(ω)

hb = Vector{Float64}(undef, N)
hole_burning!(hb, abs2gain, ω, a², [abs2.(ψ[m]) for m = 1:M])
@test all(hb .> 1)
@test hb ≈ 1 .+ abs2.(Ψ) * (abs2gain.(ω) .* a²)

end  # @testset "gain"
